﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO.Ports;

namespace WindowsFormsApplication1
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        delegate void updateStatusTextDelegate(SerialPort sp);
        void updateStatusText(SerialPort sp)
        {
            if (lblStatus.InvokeRequired)
                lblStatus.BeginInvoke(new updateStatusTextDelegate(updateStatusText), sp);
            else
                lblStatus.Text = string.Format("Port Status\r\nBreak: {0}\r\nCts: {1}\r\nDsr: {2}\r\nCD: {3}", sp.BreakState, sp.CtsHolding, sp.DsrHolding, sp.CDHolding);
        }

        private void serialPort1_PinChanged(object sender, System.IO.Ports.SerialPinChangedEventArgs e)
        {
            SerialPort sp = sender as SerialPort;

            // just update the form statuses... this is just for a demo
            updateStatusText(sp);

            // OK, so something's happened... we can check for a change using e.EventType and/or we can check for current status using the 
            // appropriate property of our serialport object sp.CtsHolding

            // Your code to do something might appear a little bit like the following:
            if (e.EventType == System.IO.Ports.SerialPinChange.Break)                 // RXD changed
            {
                if (sp.BreakState)
                {
                    // RXD has 5V
                }
                else
                {
                    // RXD has 0V or is not connected
                }                
            }
            else if (e.EventType == System.IO.Ports.SerialPinChange.CDChanged)
            {
                if (sp.CDHolding)
                {
                    // CD active (not sure what level this is - feel free to experiment)
                }
                else
                {
                    // CD other state
                }
            }
            else if (e.EventType == SerialPinChange.CtsChanged)
            {
                if (sp.CtsHolding)
                {
                    // cts 0V
                }
                else
                {
                    // cts 5V
                }
            }
            else if (e.EventType == SerialPinChange.DsrChanged)
            {
                if (sp.DsrHolding)
                {
                    // dtr 0V
                }
                else
                {
                    // dtr 5V
                }
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            foreach (string portName in SerialPort.GetPortNames())
                cboPortName.Items.Add(portName);

            if (cboPortName.Items.Count > 0)
                cboPortName.SelectedIndex = 0;
        }

        private void btnOpen_Click(object sender, EventArgs e)
        {
            try
            {
                if (serialPort1.IsOpen)
                    serialPort1.Close();

                serialPort1.PortName = cboPortName.SelectedItem.ToString();
                serialPort1.Open();
                updateStatusText(serialPort1);
            }
            catch (Exception x)
            {
                MessageBox.Show(this, string.Format("There was an error somewhere while trying to open the port\r\n{0}", x.Message));
            }
        }
    }
}
